==> pyltp-0.1.3.tar.gz: pyltp-0.1.3/ltp/src/utils/chartypes.hpp <==
#ifndef __LTP_STRUTILS_CHARTYPES_HPP__
#define __LTP_STRUTILS_CHARTYPES_HPP__

#include <cstring>
#include "chartypes.tab"
#include "unordered_map.hpp"
#include "hasher.hpp"

namespace ltp {
namespace strutils {
namespace chartypes {

enum{
  // level 1
  CHAR_LETTER = 1,
  CHAR_DIGIT = 2,
  CHAR_PUNC = 3,
  CHAR_OTHER = 0,

  // level 2
  CHAR_LETTER_SBC = 11,
  CHAR_LETTER_DBC = 12,
  CHAR_DIGIT_SBC = 21,
  CHAR_DIGIT_DBC = 22,
  CHAR_PUNC_SBC = 31,
  CHAR_PUNC_DBC = 32,

  // level 3
  CHAR_LETTER_SBC_UPPERCASE = 111,
  CHAR_LETTER_SBC_LOWERCASE = 112,
  CHAR_LETTER_DBC_UPPERCASE = 121,
  CHAR_LETTER_DBC_LOWERCASE = 122,
  CHAR_DIGIT_DBC_CL1 = 221,
  CHAR_DIGIT_DBC_CL2 = 222,
  CHAR_DIGIT_DBC_CL3 = 223,
  CHAR_PUNC_DBC_NORMAL = 321,
  CHAR_PUNC_DBC_CHINESE = 322,
  CHAR_PUNC_DBC_EXT = 323,
};


// chartype dictionary
// it's a singleton of key-value structure
template<typename T>
class __chartype_collections {
public:
  static __chartype_collections * get_collections() {
    if (0 == instance_) {
      instance_ = new __chartype_collections;
    }
    return instance_;
  }

  int chartype(const char * key) {
    internal_collection_t::const_iterator itx = collections.find(key);
    if (itx != collections.end()) {
      return itx->second;
    }
    return CHAR_OTHER;
  }

protected:
#define SETUP_TABLE(name, flag) do { \
  buff = (__chartype_##name##_buff__); \
  for (int i = 0; i < (__chartype_##name##_size__); ++ i) { \
    collections[buff] = (CHAR_##flag); \
    do {++ buff; } while (*(buff- 1)); \
  } \
} while (0);

  __chartype_collections() {
    const char * buff = 0;
    SETUP_TABLE(dbc_punc_ext_utf8,      PUNC);
    SETUP_TABLE(dbc_chinese_punc_utf8,  PUNC);
    SETUP_TABLE(dbc_digit_utf8,         DIGIT);
    SETUP_TABLE(dbc_punc_utf8,          PUNC);
    SETUP_TABLE(dbc_uppercase_utf8,     LETTER);
    SETUP_TABLE(dbc_lowercase_utf8,     LETTER);
    SETUP_TABLE(sbc_uppercase_utf8,     LETTER);
    SETUP_TABLE(sbc_digit_utf8,         DIGIT);
    SETUP_TABLE(sbc_punc_utf8,          PUNC);
    SETUP_TABLE(sbc_lowercase_utf8,     LETTER);
  }

private:
#if defined(_MSC_VER)
  typedef stdext::hash_map<const char *, int,
        utility::__Default_CharArray_HashFunction> internal_collection_t;
#else
  typedef std::tr1::unordered_map<const char *, int,
        utility::__Default_CharArray_HashFunction,
        utility::__Default_CharArray_EqualFunction> internal_collection_t;
#endif // end for _WIN32

  static __chartype_collections * instance_;
  internal_collection_t  collections;
};

template<typename T> __chartype_collections<T>* __chartype_collections<T>::instance_ = 0;

/**
 * Get the chartype for a string.
 * throughput: 42,000 calls/ms
 *
 *  @param[in]  ch  The input string.
 *  @return     int The chartype.
 */
inline int chartype(const std::string& ch) {
  return __chartype_collections<void>::get_collections()->chartype(ch.c_str());
}

/**
 * Another wrapper for const char*
 *
 *  @param[in]  ch  The input string.
 *  @return     int The chartype.
 */
inline int chartype(const char* ch) {
  return __chartype_collections<void>::get_collections()->chartype(ch);
}

}       //  end for namespace chartypes
}       //  end for namespace strutils
}       //  end for namespace ltp

#endif  //  end for __LTP_STRUTILS_CHARTYPES_HPP__

==> pyltp-0.1.5.tar.gz: pyltp-0.1.5/ltp/src/utils/chartypes.hpp <==
#ifndef __LTP_STRUTILS_CHARTYPES_HPP__
#define __LTP_STRUTILS_CHARTYPES_HPP__

#include <cstring>
#include "chartypes.tab"
#include "unordered_map.hpp"
#include "hasher.hpp"

namespace ltp {
namespace strutils {
namespace chartypes {

enum{
  // level 1
  CHAR_LETTER = 1,
  CHAR_DIGIT = 2,
  CHAR_PUNC = 3,
  CHAR_OTHER = 0,

  // level 2
  CHAR_LETTER_SBC = 11,
  CHAR_LETTER_DBC = 12,
  CHAR_DIGIT_SBC = 21,
  CHAR_DIGIT_DBC = 22,
  CHAR_PUNC_SBC = 31,
  CHAR_PUNC_DBC = 32,

  // level 3
  CHAR_LETTER_SBC_UPPERCASE = 111,
  CHAR_LETTER_SBC_LOWERCASE = 112,
  CHAR_LETTER_DBC_UPPERCASE = 121,
  CHAR_LETTER_DBC_LOWERCASE = 122,
  CHAR_DIGIT_DBC_CL1 = 221,
  CHAR_DIGIT_DBC_CL2 = 222,
  CHAR_DIGIT_DBC_CL3 = 223,
  CHAR_PUNC_DBC_NORMAL = 321,
  CHAR_PUNC_DBC_CHINESE = 322,
  CHAR_PUNC_DBC_EXT = 323,
};


// chartype dictionary
// it's a singleton of key-value structure
template<typename T>
class __chartype_collections {
public:
  static __chartype_collections * get_collections() {
    if (0 == instance_) {
      instance_ = new __chartype_collections;
    }
    return instance_;
  }

  int chartype(const char * key) {
    internal_collection_t::const_iterator itx = collections.find(key);
    if (itx != collections.end()) {
      return itx->second;
    }
    return CHAR_OTHER;
  }

protected:
#define SETUP_TABLE(name, flag) do { \
  buff = (__chartype_##name##_buff__); \
  for (int i = 0; i < (__chartype_##name##_size__); ++ i) { \
    collections[buff] = (CHAR_##flag); \
    do {++ buff; } while (*(buff- 1)); \
  } \
} while (0);

  __chartype_collections() {
    const char * buff = 0;
    SETUP_TABLE(dbc_punc_ext_utf8,      PUNC);
    SETUP_TABLE(dbc_chinese_punc_utf8,  PUNC);
    SETUP_TABLE(dbc_digit_utf8,         DIGIT);
    SETUP_TABLE(dbc_punc_utf8,          PUNC);
    SETUP_TABLE(dbc_uppercase_utf8,     LETTER);
    SETUP_TABLE(dbc_lowercase_utf8,     LETTER);
    SETUP_TABLE(sbc_uppercase_utf8,     LETTER);
    SETUP_TABLE(sbc_digit_utf8,         DIGIT);
    SETUP_TABLE(sbc_punc_utf8,          PUNC);
    SETUP_TABLE(sbc_lowercase_utf8,     LETTER);
  }

private:
  typedef std::unordered_map<const char *, int,
        utility::__Default_CharArray_HashFunction,
        utility::__Default_CharArray_EqualFunction> internal_collection_t;

  static __chartype_collections * instance_;
  internal_collection_t  collections;
};

template<typename T> __chartype_collections<T>* __chartype_collections<T>::instance_ = 0;

/**
 * Get the chartype for a string.
 * throughput: 42,000 calls/ms
 *
 *  @param[in]  ch  The input string.
 *  @return     int The chartype.
 */
inline int chartype(const std::string& ch) {
  return __chartype_collections<void>::get_collections()->chartype(ch.c_str());
}

/**
 * Another wrapper for const char*
 *
 *  @param[in]  ch  The input string.
 *  @return     int The chartype.
 */
inline int chartype(const char* ch) {
  return __chartype_collections<void>::get_collections()->chartype(ch);
}

}       //  end for namespace chartypes
}       //  end for namespace strutils
}       //  end for namespace ltp

#endif  //  end for __LTP_STRUTILS_CHARTYPES_HPP__
